@@ -5,12 +5,12 @@ from abc import abstractmethod, ABCMeta
 from threading import Timer, Lock
 
 from optimeed.core.tools import getPath_workspace
-from .myjson import obj_to_json, json_to_obj, json_to_obj_safe, encode_str_json, decode_str_json, get_json_module_tree_from_dict, MODULE_TAG, CLASS_TAG
+from .myjson import obj_to_json, json_to_obj, json_to_obj_safe, encode_str_json, decode_str_json, get_json_module_tree_from_dict, remove_module_tree_from_string, apply_module_tree_to_dict
 from .tools import indentParagraph, rgetattr, printIfShown, SHOW_WARNING, SHOW_DEBUG, SHOW_INFO, SHOW_ERROR, delete_indices_from_list
 import re
 import inspect
 from shutil import rmtree
-from math import floor
+from math import floor, ceil
 import linecache
 
 
@@ -93,20 +93,6 @@ class ListDataStruct_Interface(DataStruct_Interface, metaclass=ABCMeta):
         """
         pass
 
-    @staticmethod
-    def _recursive_apply_moduleTree(nestedTree, nestedObject, raiseError=False):
-        for key in nestedTree:
-            if key == CLASS_TAG or key == MODULE_TAG:
-                try:
-                    nestedObject[key] = nestedTree[key]
-                except TypeError as e:
-                    printIfShown("(following error when loading, might come from numpy elementary types or 'null' values) {}".format(nestedTree, e), SHOW_DEBUG)
-                    if raiseError:
-                        raise
-            else:
-                if isinstance(nestedObject, dict):
-                    ListDataStruct._recursive_apply_moduleTree(nestedTree[key], nestedObject[key])
-
     @staticmethod
     def _jsondata_to_obj(item, theClass=None):
         return json_to_obj(item) if theClass is None else json_to_obj_safe(item, theClass)
@@ -226,7 +212,7 @@ class ListDataStruct(ListDataStruct_Interface):
         return len(self.theData)
 
     def clone(self, filename):
-        self.save(filename)  # Same behaviour as save: all the memory is dumped
+        self.save(filename)  # Same behaviour as 'save': all the memory is dumped
 
     def save(self, filename):
         """Save data using json format. The data to be saved are automatically detected, see :meth:`~optimeed.core.myjson.obj_to_json` """
@@ -268,11 +254,9 @@ class ListDataStruct(ListDataStruct_Interface):
             obj_str += "\t" + json.dumps(obj_to_json(item))
             # if compress save: remove all information relative to module and class
             if self.compress_save:
-                obj_str = re.sub("\s*\"__module__\" *: *(\"(.*?)\"(,|\s|)|\s*\{(.*?)\}(,|\s|))", '', obj_str)
-                obj_str = re.sub("\s*\"__class__\" *: *(\"(.*?)\"(,|\s|)|\s*\{(.*?)\}(,|\s|))", '', obj_str)
+                remove_module_tree_from_string(obj_str)
 
             listStr += obj_str
-
             has_previous_element = True
         listStr += "\n]"
 
@@ -306,7 +290,7 @@ class ListDataStruct(ListDataStruct_Interface):
                 try:
                     theModuleTree = theDict[ListDataStruct._COMPRESS_SAVE_STR]
                     for item in theDict[ListDataStruct._DATA_STR]:
-                        ListDataStruct._recursive_apply_moduleTree(theModuleTree, item)
+                        apply_module_tree_to_dict(theModuleTree, item)
                 except KeyError:
                     pass
 
@@ -485,16 +469,29 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
         return nbr_elements_full_files + nbr_elements_last_file + unsaved_elements
 
     def add_data(self, theData):
+        """Add data to the collection"""
         theDict = obj_to_json(theData)
         theStr = json.dumps(theDict)
         if self.compress_save:
-            theStr = re.sub("\s*\"__module__\" *: *(\"(.*?)\"(,|\s|)|\s*\{(.*?)\}(,|\s|))", '', theStr)
-            theStr = re.sub("\s*\"__class__\" *: *(\"(.*?)\"(,|\s|)|\s*\{(.*?)\}(,|\s|))", '', theStr)
+            theStr = remove_module_tree_from_string(theStr)
             self._save_moduletree(theDict)
         global theLock
         with theLock:
             self.data_buffer.append(theStr)
 
+    def add_json_data(self, theStr):
+        """Add already deserialized data to the collection"""
+        if self.compress_save:
+            # If class does not have module tree => Get it
+            if not self.json_module_tree:
+                theDict = json.loads(theStr)
+                self._save_moduletree(theDict)
+            theStr = remove_module_tree_from_string(theStr)
+
+        global theLock
+        with theLock:
+            self.data_buffer.append(theStr)
+
     def _save_moduletree(self, theDict):
         if not self.json_module_tree:
             if not theDict == "null":
@@ -520,8 +517,7 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
         return json_str
 
     def reorder(self, permutations):
-        """
-        Reorder collection accordingly to permutations.
+        """Reorder collection accordingly to permutations.
         E.G, list_of_indices = [0,3,2] with collection elems [0,2,1] => collection elems = [0,2,3]
         :param permutations:
         :return: /
@@ -538,22 +534,21 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
         self.has_been_initialized = False
 
     def get_data_at_index(self, index, ignore_attributes=None, none_if_error=False):
+        """Same as parent, with additional kwargs
+
+        :param index:
+        :param ignore_attributes: ignore attributes to deserialize (list)
+        :param none_if_error:
+        :return:
         """
 
-        Args:
-            index: index to get
-            ignore_attributes: ignore attributes to deserialize (list)
-
-        Returns:
-
-        """
         try:
             json_str = self._get_json_str_at_index(index)
             if not json_str:
                 json_str = self._get_json_str_at_index(index, refresh_cache=True)
 
             item = json.loads(json_str)
-            ListDataStruct._recursive_apply_moduleTree(json.loads(self.json_module_tree), item, raiseError=none_if_error)
+            apply_module_tree_to_dict(json.loads(self.json_module_tree), item, raiseError=none_if_error)
 
             if ignore_attributes is not None:
                 for attribute in ignore_attributes:
@@ -604,7 +599,7 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
         newStruct.stack_size = theDict[Performance_ListDataStruct._STACK_SIZE]
 
         nbr_elements = theDict[Performance_ListDataStruct._NBR_ELEMENTS]
-        number_of_files = int(floor(nbr_elements/newStruct.stack_size)) + 1
+        number_of_files = int(ceil(nbr_elements/newStruct.stack_size))
         number_of_elements_last_file = nbr_elements - (number_of_files-1)*newStruct.stack_size
 
         newStruct.stack_size = theDict[Performance_ListDataStruct._STACK_SIZE]
@@ -617,16 +612,14 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
 
     @staticmethod
     def load_as_listdatastruct(filename, **kwargs):
-        """Return :class:`ListDataStruct`. Two-steps:
-        1) Save temporary file formated like ListDataStruct
-        2) Use ListDataStruct load function"""
-        # 1)
+        """Return :class:`ListDataStruct`. Serializes all objects of collection"""
+        # 1) Save temporary file formated like ListDataStruct
         with open(filename, 'r', encoding="utf-8", newline='\n') as f:
             theDict = json.load(f)
 
         stack_size = theDict[Performance_ListDataStruct._STACK_SIZE]
         nbr_elements = theDict[Performance_ListDataStruct._NBR_ELEMENTS]
-        number_of_files = int(floor(nbr_elements/stack_size)) + 1
+        number_of_files = int(ceil(nbr_elements/stack_size))
         lines = list()
         for file_number in range(number_of_files):
             with open(Performance_ListDataStruct._get_path_from_filenumber(filename, file_number), 'r', encoding='utf-8', errors='ignore') as f:
@@ -652,14 +645,13 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
         tempfile = filename + "_temp"
         with open(tempfile, "w", encoding="utf-8") as f:
             f.write(theStr)
-        # 2)
+        # 2) Use ListDataStruct load function
         theDataStruct = ListDataStruct.load(tempfile, **kwargs)
         os.remove(tempfile)
         printIfShown("Done loading", SHOW_INFO)
         return theDataStruct
 
     def get_data_generator(self, **kwargs):
-        """Get a generator to all the data stored"""
         for k in range(self.get_total_nbr_elements(count_unsaved=True)):
             yield self.get_data_at_index(k, kwargs)
 
@@ -671,8 +663,7 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
         self.set_data_at_indices([data_in], [index])
 
     def set_data_at_indices(self, data_list, indices):
-        """
-        Replace datas at specific indices
+        """Replace datas at specific indices
         :param data_list: list of objects to set to the collection, at specific indices
         :param indices: list of indices
         :return:
@@ -684,7 +675,7 @@ class Performance_ListDataStruct(ListDataStruct_Interface):
         else:
             if len(self.data_buffer):
                 printIfShown("Buffer of collection is not empty. Please use save function first !", SHOW_ERROR)
-            # First, convert data to something readible
+            # First, convert data to something readable
             self.data_buffer = list()
             for data in data_list:
                 self.add_data(data)
