@@ -8,6 +8,7 @@ SHOW_CURRENT = [SHOW_INFO, SHOW_WARNING, SHOW_ERROR, SHOW_DEBUG, SHOW_LOGS]
 
 
 def setCurrentShow(show_types):
+    """Change text type to be displayed by PrintIfShown"""
     global SHOW_CURRENT
     SHOW_CURRENT.clear()
     for t in show_types:
@@ -16,11 +17,13 @@ def setCurrentShow(show_types):
 
 
 def getCurrentShow():
+    """Get text type to be displayed by PrintIfShown"""
     global SHOW_CURRENT
     return list(SHOW_CURRENT)
 
 
 def disableLogs():
+    """Disable all logs"""
     global SHOW_CURRENT
     try:
         SHOW_CURRENT.remove(SHOW_LOGS)
@@ -29,6 +32,7 @@ def disableLogs():
 
 
 def enableLogs():
+    """Show all logs"""
     global SHOW_CURRENT
     if SHOW_LOGS not in SHOW_CURRENT:
         SHOW_CURRENT.append(SHOW_LOGS)
