@@ -63,6 +63,7 @@ class Data:
         self.meta = meta
 
     def set_kwargs(self, kwargs):
+        """Set a kwarg after creation of the class"""
         for kwarg in kwargs:
             setattr(self, kwarg, kwargs[kwarg])
 
@@ -72,6 +73,7 @@ class Data:
         self.y = y
 
     def set_meta(self, meta):
+        """Set associated 'Z' data"""
         self.meta = meta
 
     def get_x(self):
@@ -110,6 +112,7 @@ class Data:
         return self.y
 
     def get_meta(self):
+        """Get associated 'Z' data"""
         if self.meta is None:
             return None
         if len(self.get_y()) == len(self.meta):
@@ -125,6 +128,7 @@ class Data:
         return convert_color_with_alpha(self.color, self.alpha)
 
     def get_alpha(self):
+        """Get opacity"""
         return self.alpha
 
     def get_width(self):
@@ -321,9 +325,11 @@ class Data:
         return theStr
 
     def set_color(self, theColor):
+        """Set trace color"""
         self.color = theColor
 
     def set_legend(self, theLegend):
+        """Set legend"""
         self.legend = theLegend
 
 
