@@ -3,4 +3,5 @@ from .mathsToPhysics import *
 from .objAndCons import *
 from .optiAlgorithms import *
 from .optiVariable import Real_OptimizationVariable, Binary_OptimizationVariable, Integer_OptimizationVariable
-from .optimizer import Optimizer
+from .optimizer import run_optimization, OptimizerSettings
+from .optiHistoric import OptiHistoric
