@@ -15,7 +15,7 @@ class InterfaceObjCons(metaclass=ABCMeta):
         pass
 
     def get_name(self):
-        return "Objective / constraint"
+        return "Objective (or constraint)"
 
     def __str__(self):
         return self.get_name()
